@@ -58,7 +58,7 @@ class CashKarp(Integrator):
         quit1 = self.quit1
         quit2 = self.quit2
 
-        while x[-1] <= x_end and self.stopping_criterion(*y[-1]):
+        while abs(x[-1]) <= abs(x_end) and self.stopping_criterion(*y[-1]):
             if self.verbose:
                 print("{:.4f}".format(x[-1]), end= "\r")
             next = self.next_step(x[-1], y[-1], h, twiddle1, twiddle2, quit1, quit2)
